;
; djmouse.asm: Mouse driver for DJGPP
;

%define UNTOUCHED 0xffff                        ; who knows the proper value?
global  _MouseInit
global  _MouseDeinit
global  _MouseClick
global  _MouseGetHV
extern  _genesis
extern  _control

%define cache_align times ($$-$) & 31 nop       ; align to 32-byte boundary

struc genesisstruc
        .rom            resd 1
        .ram            resd 1
        .z80_ram        resd 1
        .z80_emu        resd 1
        .hint           resd 1
        .menacer        resd 1
        .version        resd 1
        .busreq         resd 1        
        .vsync          resd 1
        .pri            resd 1
        .spr_limit      resd 1
        .save_slot      resd 1
endstruc

struc controlstruc
        .quit           resd 1          ; signal quit
        .ctrl1          resd 1          ; ctrl pad 1: 0x00 0x00 low high
        .ctrl2          resd 1          ; ctrl pad 2
        .ctrl1_select   resd 1
        .ctrl2_select   resd 1
endstruc

struc vdpstruc
        .vram                   resd 1
        .cram                   resd 1
        .vsram                  resd 1
        .ctrlport_mode          resd 1
        .access_mode            resd 1
        .addr                   resd 1
        .dma_mode               resd 1
        .control_port           resd 1
        .status_register        resd 1
        .hv_counter             resd 1
        .reg                    resd 24
        .scroll_a_addr          resd 1
        .scroll_b_addr          resd 1
        .window_addr            resd 1
        .sprite_addr            resd 1
        .hscroll_addr           resd 1
        .window_x               resd 1
        .window_y               resd 1
        .window_xlimit          resd 1
        .window_ylimit          resd 1
        .sprite_order           resd 80
        .sprite_count           resd 1
        .hint_timer             resd 1
endstruc


bits 32
section .data

        old_selector    dw 0
        old_offset      dd 0


section .text

cache_align
_MouseInit:
        push    ebp
        mov     ebp, esp

        xor     eax, eax        ; reset Microsoft Mouse Driver
        int     0x33

        cmp     ax, 0
        je      .bad            ; if 0, then error

        mov     ax, 7           ; set horizontal boundaries
        mov     ecx, 0
        mov     edx, 320 - 8
        int     0x33
        mov     ax, 8           ; set vertical boundaries
        mov     ecx, 0
        mov     edx, 224 - 8
        int     0x33
        mov     ax, 0xf         ; increase sensitivity to 6 pixels per mickey
        mov     cx, 6
        mov     dx, 6
        int     0x33

        xor     eax, eax        ; no error

        ; untouched Menacer
        mov     dword [_control+controlstruc.ctrl2], UNTOUCHED

        pop     ebp
        ret

.bad:
        mov     eax, 1
        pop     ebp
        ret

cache_align
_MouseDeinit:
        push    ebp
        mov     ebp, esp

        xor     eax, eax        ; reset driver
        int     0x33

        pop     ebp
ret

cache_align
_MouseClick:                    ; return left button in bit 0, right bit 1
        push    ebp
        mov     ebp, esp

        mov     eax, 3          ; get mouse button status 
        int     0x33
        mov     al, bl
        and     eax, 3          ; bit 0 left button, bit 1 right button

        pop     ebp
ret

cache_align
_MouseGetHV:                    ; return mouse coordinates as HV counter value
        push    ebp
        mov     ebp, esp

        mov     eax, 3          ; get coordinates
        int     0x33

;        shr     ecx, 1          ; get rid of H0
        mov     ah, dl          ; V coordinate (8 bits, V7-V0)
        mov     al, cl          ; H coordinate (H8-H0)

        pop     ebp
ret
