/*
 * vga.c: VGA code
 */

#include <dpmi.h>
#include <go32.h>
#include <pc.h>
#include <stdio.h>
#include <sys/farptr.h>
#include "genital.h"
#include "vdp.h"
#include "vga.h"


void SetPalette(unsigned char color, unsigned short int rgb)
{
        outportb(PEL_WRITE_ADDR_REG, color >> 1);       /* request color */

        /* fix: optimize the shifting here */
        outportb(PEL_DATA_REG, ((rgb >> 1) & 7) << 3);  /* red */
        outportb(PEL_DATA_REG, ((rgb >> 5) & 7) << 3);  /* green */
        outportb(PEL_DATA_REG, ((rgb >> 9) & 7) << 3);  /* blue */        
}
        
void GraphicsInit()
{
        __dpmi_regs     r;
        int             i;        
        unsigned char   v;

        _farsetsel(_dos_ds);            /* now fs is the global segment reg */

        /* enter mode 13h */
        r.x.ax = 0x13;
        __dpmi_int(0x10, &r);

        /* initialize VGA so that it lets us use any register combination */
        outportb(0x3d4, 0x11);
        v = inportb(0x3d5) & 0x7f;
        outportb(0x3d4, 0x11);
        outportb(0x3d5, v);

        outportb(0x3c6, 0xff);  /* make all palette registers accesible */

        /* set all colors to 0 initially */
        for (i = 0; i < 64; i++)
        {
                outportb(PEL_WRITE_ADDR_REG, i); 
                outportb(PEL_DATA_REG, 0);
                outportb(PEL_DATA_REG, 0);
                outportb(PEL_DATA_REG, 0);
        }

        /* set border color to 255 (which will be 0) */
        outportb(ATTRIB_REG, 0x31);     /* index 0x11: border color */
        outportb(ATTRIB_REG, 255);      /* ... bit 5 was set for screen on */

        /* color 254 is used by GUI for now */
        outportb(PEL_WRITE_ADDR_REG, 254);
        outportb(PEL_DATA_REG, 0xff);
        outportb(PEL_DATA_REG, 0xff);
        outportb(PEL_DATA_REG, 0xff);
}
        
void GraphicsDeinit()
{
        __dpmi_regs     r;

        r.x.ax = 3;
        __dpmi_int(0x10, &r);
}

